<?php
	/*
	* Task_FinderFPs Plugin pour composant Finder Joomla pour Joomla! 4.x
	* Version J4_0
	* Creation date: Juin 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: support@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Plugin\Task\FinderFp\Extension;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Component\Scheduler\Administrator\Event\ExecuteTaskEvent;
use Joomla\Component\Scheduler\Administrator\Task\Status;
use Joomla\Component\Scheduler\Administrator\Task\Task;
use Joomla\Component\Scheduler\Administrator\Traits\TaskPluginTrait;
use Joomla\Database\DatabaseAwareInterface;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Event\SubscriberInterface;
use RuntimeException;

use Joomla\Component\Finder\Administrator\Indexer\Indexer;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Event\Finder\StartIndexEvent;
use Joomla\CMS\Event\Finder\BeforeIndexEvent;
use Joomla\CMS\Event\Finder\BuildIndexEvent;
use Joomla\Component\Finder\Administrator\Response\Response;
use Joomla\CMS\Language\Text;


class FinderFp extends CMSPlugin implements SubscriberInterface, DatabaseAwareInterface
{
	use DatabaseAwareTrait;
	use TaskPluginTrait;

	private const TASKS_MAP = [
		'finderfp.update' => [
			'langConstPrefix' => 'PLG_TASK_FINDERFP_TASK_UPDATE',
			'method'          => 'takeUpdate',
		],
	];

	protected $autoloadLanguage = true;

	public static function getSubscribedEvents(): array
	{
		// This task is disabled if the Akeeba Backup component is not installed or has been unpublished
		if (!ComponentHelper::isEnabled('com_finder'))
		{
			return [];
		}

		return [
			'onTaskOptionsList'    => 'advertiseRoutines',
			'onExecuteTask'        => 'standardRoutineHandler',
		];
	}

	//regénère l'index du com_finder
	private function takeUpdate(ExecuteTaskEvent $event): int
	{
        // Get some basic information about the task at hand.
		/** @var Task $task */
		$task    = $event->getArgument('subject');

		if (!ComponentHelper::isEnabled('com_finder'))
		{
			throw new RuntimeException('The Finder component is not installed or has been disabled.');
		}

		$this->logTask('Starting a new indexation.');
		
		$return = json_decode($this->start());
		if ($return->totalItems != 0) $continu = true;
		else $continu = false;
		
		while ($continu)
		{
			$this->logTask('Starting a new batch');
			$return = json_decode($this->batch());
			if ($return->totalItems == 0) $continu = false;
		}
		
		// Did the backup end in an error?
		// if (isset($updateResult['Error']) && !empty($updateResult['Error']))
		// {
			// throw new RuntimeException($updateResult['Error']);
		// }

		$this->logTask('Finish indexation successfully');

		return Status::OK;
	}
	
	private function start() {
        $app = Factory::getApplication();
		
		// $dispatcher = $app->getDispatcher();

        // We don't want this form to be cached.
        $app->allowCache(false);

        // Put in a buffer to silence noise.
        ob_start();

        // Reset the indexer state.
        Indexer::resetState();

        // Import the finder plugins.
        // PluginHelper::importPlugin('finder', null, true, $dispatcher);
        PluginHelper::importPlugin('finder');

        // Add the indexer language to \JS
        Text::script('COM_FINDER_AN_ERROR_HAS_OCCURRED');
        Text::script('COM_FINDER_NO_ERROR_RETURNED');

        // Start the indexer.
        try {
            // Trigger the onStartIndex event.
            // $dispatcher->dispatch('onStartIndex', new StartIndexEvent('onStartIndex', []));
            $app->triggerEvent('onStartIndex');

            // Get the indexer state.
            $state        = Indexer::getState();
            $state->start = 1;

            $output = ob_get_contents();

            // Finder plugins should not create output of any kind. If there is output, that very likely is the result of a PHP error.
            if (trim($output)) {
                throw new \Exception(Text::_('COM_FINDER_AN_ERROR_HAS_OCCURRED'));
            }

            // Send the response.
            return static::sendResponse($state);
        } catch (\Exception $e) {
            // Catch an exception and return the response.
            return static::sendResponse($e);
        }
	}
	
    private function batch()
    {
        $app = Factory::getApplication();

        // $dispatcher = $app->getDispatcher();

        // We don't want this form to be cached.
        $app->allowCache(false);

        // Put in a buffer to silence noise.
        ob_start();

        // Remove the script time limit.
        if (\function_exists('set_time_limit')) {
            set_time_limit(0);
        }

        // Get the indexer state.
        $state = Indexer::getState();

        // Reset the batch offset.
        $state->batchOffset = 0;

        // Update the indexer state.
        Indexer::setState($state);

        // Import the finder plugins.
        // PluginHelper::importPlugin('finder', null, true, $dispatcher);
        PluginHelper::importPlugin('finder');

        /*
         * We are going to swap out the raw document object with an HTML document
         * in order to work around some plugins that don't do proper environment
         * checks before trying to use HTML document functions.
         */
        $lang = $app->getLanguage();

        // Get the document properties.
        $attributes = [
            'charset'   => 'utf-8',
            'lineend'   => 'unix',
            'tab'       => '  ',
            'language'  => $lang->getTag(),
            'direction' => $lang->isRtl() ? 'rtl' : 'ltr',
        ];

        // Start the indexer.
        try {
            // Trigger the onBeforeIndex event.
            // $dispatcher->dispatch('onBeforeIndex', new BeforeIndexEvent('onBeforeIndex', []));
            $app->triggerEvent('onBeforeIndex');

            // Trigger the onBuildIndex event.
            // $dispatcher->dispatch('onBuildIndex', new BuildIndexEvent('onBuildIndex', []));
            $app->triggerEvent('onBuildIndex');

            // Get the indexer state.
            // $state           = Indexer::getState();
            $state->start    = 0;
            $state->complete = 0;

            // $output = ob_get_contents();

            // Finder plugins should not create output of any kind. If there is output, that very likely is the result of a PHP error.
            // if (trim($output)) {
                // throw new \Exception(Text::_('COM_FINDER_INDEXER_ERROR_PLUGIN_FAILURE'));
            // }

            // Send the response.
            return static::sendResponse($state);
        } catch (\Exception $e) {
            // Catch an exception and return the response.
            // Send the response.
            return static::sendResponse($e);
        }
    }
	
    public static function sendResponse($data = null)
    {
        $app = Factory::getApplication();

        // Send the assigned error code if we are catching an exception.
        if ($data instanceof \Exception) {
            $app->setHeader('status', $data->getCode());
        }

        // Create the response object.
        $response = new Response($data);

        ob_clean();

        // Send the JSON response.
        return json_encode($response);
    }
}